@@ -17,6 +17,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <arpa/inet.h>
+#include <time.h>
 #include <errno.h>
 
 #include <libnfnetlink/libnfnetlink.h>
@@ -26,13 +27,19 @@
 #define IPPROTO_SCTP 132
 #endif
 
+#ifndef IPPROTO_UDPLITE
+#define IPPROTO_UDPLITE 136
+#endif
+
 struct nfct_handle;
 
 typedef void (*set_attr)(struct nf_conntrack *ct, const void *value);
 typedef const void *(*get_attr)(const struct nf_conntrack *ct);
+typedef void (*copy_attr)(struct nf_conntrack *d, const struct nf_conntrack *o);
 
 extern set_attr set_attr_array[];
 extern get_attr get_attr_array[];
+extern copy_attr copy_attr_array[];
 
 typedef int (*nfct_handler)(struct nfct_handle *cth, struct nlmsghdr *nlh,
 			    void *arg);
@@ -55,7 +62,24 @@ struct nfct_handle {
 			void *data);
 };
 
-union __nfct_l4 {
+union __nfct_l4_src {
+	/* Add other protocols here. */
+	u_int16_t all;
+	struct {
+		u_int16_t port;
+	} tcp;
+	struct {
+		u_int16_t port;
+	} udp;
+	struct {
+		u_int16_t id;
+	} icmp;
+	struct {
+		u_int16_t port;
+	} sctp;
+};
+
+union __nfct_l4_dst {
 	/* Add other protocols here. */
 	u_int16_t all;
 	struct {
@@ -66,7 +90,6 @@ union __nfct_l4 {
 	} udp;
 	struct {
 		u_int8_t type, code;
-		u_int16_t id;
 	} icmp;
 	struct {
 		u_int16_t port;
@@ -84,14 +107,34 @@ struct __nfct_tuple {
 
 	u_int8_t l3protonum;
 	u_int8_t protonum;
-	union __nfct_l4 l4src;
-	union __nfct_l4 l4dst;
+	union __nfct_l4_src l4src;
+	union __nfct_l4_dst l4dst;
+
+	struct {
+		u_int32_t correction_pos;
+		u_int32_t offset_before;
+		u_int32_t offset_after;
+	} natseq;
 };
 
+#define __DIR_ORIG 0
+#define __DIR_REPL 1
+#define __DIR_MASTER 2
+#define __DIR_MAX __DIR_MASTER+1
+
 union __nfct_protoinfo {
 	struct {
 		u_int8_t state;
+		struct {
+			u_int8_t value;
+			u_int8_t mask;
+		} flags[__DIR_MAX];
 	} tcp;
+	struct {
+		u_int8_t state;
+		u_int32_t vtag[__DIR_MAX];
+	} sctp;
+
 };
 
 struct __nfct_counters {
@@ -101,18 +144,15 @@ struct __nfct_counters {
 
 struct __nfct_nat {
 	u_int32_t min_ip, max_ip;
-	union __nfct_l4 l4min, l4max;
+	union __nfct_l4_src l4min, l4max;
 };
 
-#define __DIR_ORIG 0
-#define __DIR_REPL 1
-#define __DIR_MAX __DIR_REPL+1
-
 struct nf_conntrack {
 	struct __nfct_tuple tuple[__DIR_MAX];
 	
 	u_int32_t 	timeout;
 	u_int32_t	mark;
+	u_int32_t	secmark;
 	u_int32_t 	status;
 	u_int32_t	use;
 	u_int32_t	id;
@@ -173,6 +213,7 @@ void __parse_tuple(const struct nfattr *attr, struct __nfct_tuple *tuple, int di
 int __snprintf_conntrack(char *buf, unsigned int len, const struct nf_conntrack *ct, unsigned int type, unsigned int msg_output, unsigned int flags);
 int __snprintf_address(char *buf, unsigned int len, const struct __nfct_tuple *tuple);
 int __snprintf_protocol(char *buf, unsigned int len, const struct nf_conntrack *ct);
+int __snprintf_proto(char *buf, unsigned int len, const struct __nfct_tuple *tuple);
 int __snprintf_conntrack_default(char *buf, unsigned int len, const struct nf_conntrack *ct, const unsigned int msg_type, const unsigned int flags);
 int __snprintf_conntrack_xml(char *buf, unsigned int len, const struct nf_conntrack *ct, const unsigned int msg_type, const unsigned int flags);
 
@@ -181,7 +222,7 @@ int __callback(struct nlmsghdr *nlh, struct nfattr *nfa[], void *data);
 
 int __setobjopt(struct nf_conntrack *ct, unsigned int option);
 int __getobjopt(const struct nf_conntrack *ct, unsigned int option);
-int __compare(const struct nf_conntrack *ct1, const struct nf_conntrack *ct2);
+int __compare(const struct nf_conntrack *ct1, const struct nf_conntrack *ct2, unsigned int flags);
 
 typedef void (*set_exp_attr)(struct nf_expect *exp, const void *value);
 typedef const void *(*get_exp_attr)(const struct nf_expect *exp);
@@ -193,5 +234,38 @@ int __build_expect(struct nfnl_subsys_handle *ssh, struct nfnlhdr *req, size_t s
 int __parse_expect_message_type(const struct nlmsghdr *nlh);
 void __parse_expect(const struct nlmsghdr *nlh, const struct nfattr *cda[], struct nf_expect *exp);
 int __expect_callback(struct nlmsghdr *nlh, struct nfattr *nfa[], void *data);
+int __snprintf_expect(char *buf, unsigned int len, const struct nf_expect *exp, unsigned int type, unsigned int msg_output, unsigned int flags);
+int __snprintf_expect_default(char *buf, unsigned int len, const struct nf_expect *exp, unsigned int msg_type, unsigned int flags);
+
+/* backward compatibility of the deprecated API */
+extern struct nfct_l3proto ipv4;
+extern struct nfct_l3proto ipv6;
+
+extern struct nfct_proto tcp;
+extern struct nfct_proto udp;
+extern struct nfct_proto sctp;
+extern struct nfct_proto icmp;
+
+#define TS_ORIG								\
+({									\
+	((1 << ATTR_ORIG_IPV4_SRC) | (1 << ATTR_ORIG_IPV4_DST) |	\
+	 (1 << ATTR_ORIG_IPV6_SRC) | (1 << ATTR_ORIG_IPV6_DST) |	\
+	 (1 << ATTR_ORIG_PORT_SRC) | (1 << ATTR_ORIG_PORT_DST) | 	\
+	 (1 << ATTR_ORIG_L3PROTO)  | (1 << ATTR_ORIG_L4PROTO)  | 	\
+	 (1 << ATTR_ICMP_TYPE)	   | (1 << ATTR_ICMP_CODE)     | 	\
+	 (1 << ATTR_ICMP_ID));						\
+})
+
+#define TS_REPL								\
+({									\
+	((1 << ATTR_REPL_IPV4_SRC) | (1 << ATTR_REPL_IPV4_DST) | 	\
+	 (1 << ATTR_REPL_IPV6_SRC) | (1 << ATTR_REPL_IPV6_DST) | 	\
+	 (1 << ATTR_REPL_PORT_SRC) | (1 << ATTR_REPL_PORT_DST) | 	\
+	 (1 << ATTR_REPL_L3PROTO)  | (1 << ATTR_REPL_L4PROTO)  |	\
+	 (1 << ATTR_ICMP_TYPE)	   | (1 << ATTR_ICMP_CODE)     | 	\
+	 (1 << ATTR_ICMP_ID));						\
+})
+
+#define TUPLE_SET(dir) (dir == __DIR_ORIG ? TS_ORIG : TS_REPL)
 
 #endif
