@@ -1,5 +1,5 @@
 /*
- * (C) 2005-2007 by Pablo Neira Ayuso <pablo@netfilter.org>
+ * (C) 2005-2008 by Pablo Neira Ayuso <pablo@netfilter.org>
  *
  * This software may be used and distributed according to the terms
  * of the GNU General Public License, incorporated herein by reference.
@@ -98,6 +98,28 @@ enum nf_conntrack_attr {
 	ATTR_USE,				/* u32 bits */
 	ATTR_ID,				/* u32 bits */
 	ATTR_STATUS = 32,			/* u32 bits  */
+	ATTR_TCP_FLAGS_ORIG,			/* u8 bits */
+	ATTR_TCP_FLAGS_REPL,			/* u8 bits */
+	ATTR_TCP_MASK_ORIG,			/* u8 bits */
+	ATTR_TCP_MASK_REPL = 36,		/* u8 bits */
+	ATTR_MASTER_IPV4_SRC,			/* u32 bits */
+	ATTR_MASTER_IPV4_DST,			/* u32 bits */
+	ATTR_MASTER_IPV6_SRC,			/* u128 bits */
+	ATTR_MASTER_IPV6_DST = 40,		/* u128 bits */
+	ATTR_MASTER_PORT_SRC,			/* u16 bits */
+	ATTR_MASTER_PORT_DST,			/* u16 bits */
+	ATTR_MASTER_L3PROTO,			/* u8 bits */
+	ATTR_MASTER_L4PROTO = 44,		/* u8 bits */
+	ATTR_SECMARK,				/* u32 bits */
+	ATTR_ORIG_NAT_SEQ_CORRECTION_POS,	/* u32 bits */
+	ATTR_ORIG_NAT_SEQ_OFFSET_BEFORE,	/* u32 bits */
+	ATTR_ORIG_NAT_SEQ_OFFSET_AFTER = 48,	/* u32 bits */
+	ATTR_REPL_NAT_SEQ_CORRECTION_POS,	/* u32 bits */
+	ATTR_REPL_NAT_SEQ_OFFSET_BEFORE,	/* u32 bits */
+	ATTR_REPL_NAT_SEQ_OFFSET_AFTER,		/* u32 bits */
+	ATTR_SCTP_STATE = 52,			/* u8 bits */
+	ATTR_SCTP_VTAG_ORIG,			/* u32 bits */
+	ATTR_SCTP_VTAG_REPL,			/* u32 bits */
 	ATTR_MAX
 };
 
@@ -229,6 +251,12 @@ enum {
 enum {
 	NFCT_OF_SHOW_LAYER3_BIT = 0,
 	NFCT_OF_SHOW_LAYER3 = (1 << NFCT_OF_SHOW_LAYER3_BIT),
+
+	NFCT_OF_TIME_BIT = 1,
+	NFCT_OF_TIME = (1 << NFCT_OF_TIME_BIT),
+
+	NFCT_OF_ID_BIT = 2,
+	NFCT_OF_ID = (1 << NFCT_OF_ID_BIT),
 };
 
 extern int nfct_snprintf(char *buf, 
@@ -238,9 +266,26 @@ extern int nfct_snprintf(char *buf,
 			 const unsigned int out_type,
 			 const unsigned int out_flags);
 
+/* comparison */
 extern int nfct_compare(const struct nf_conntrack *ct1,
 			const struct nf_conntrack *ct2);
 
+enum {
+	NFCT_CMP_ALL = 0,
+	NFCT_CMP_ORIG = (1 << 0),
+	NFCT_CMP_REPL = (1 << 1),
+	NFCT_CMP_TIMEOUT_EQ = (1 << 2),
+	NFCT_CMP_TIMEOUT_GT = (1 << 3),
+	NFCT_CMP_TIMEOUT_GE = (NFCT_CMP_TIMEOUT_EQ | NFCT_CMP_TIMEOUT_GT),
+	NFCT_CMP_TIMEOUT_LT = (1 << 4),
+	NFCT_CMP_TIMEOUT_LE = (NFCT_CMP_TIMEOUT_EQ | NFCT_CMP_TIMEOUT_LT),
+};
+
+extern int nfct_cmp(const struct nf_conntrack *ct1,
+		    const struct nf_conntrack *ct2,
+		    unsigned int flags);
+
+
 /* query */
 enum nf_conntrack_query {
 	NFCT_Q_CREATE,
@@ -257,8 +302,28 @@ extern int nfct_query(struct nfct_handle *h,
 		      const enum nf_conntrack_query query,
 		      const void *data);
 
+extern int nfct_send(struct nfct_handle *h,
+		     const enum nf_conntrack_query query,
+		     const void *data);
+
 extern int nfct_catch(struct nfct_handle *h);
 
+/* copy */
+enum {
+	NFCT_CP_ALL = 0,
+	NFCT_CP_ORIG = (1 << 0),
+	NFCT_CP_REPL = (1 << 1),
+	NFCT_CP_META = (1 << 2),
+};
+
+extern void nfct_copy(struct nf_conntrack *dest,
+		      const struct nf_conntrack *source,
+		      unsigned int flags);
+
+extern void nfct_copy_attr(struct nf_conntrack *ct1,
+			   const struct nf_conntrack *ct2,
+			   const enum nf_conntrack_attr type);
+
 /* low level API: netlink functions */
 
 extern int nfct_build_conntrack(struct nfnl_subsys_handle *ssh,
@@ -427,6 +492,22 @@ enum ip_conntrack_status {
 	IPS_FIXED_TIMEOUT = (1 << IPS_FIXED_TIMEOUT_BIT),
 };
 
+/*
+ * TCP flags
+ */
+
+/* Window scaling is advertised by the sender */
+#define IP_CT_TCP_FLAG_WINDOW_SCALE             0x01
+
+/* SACK is permitted by the sender */
+#define IP_CT_TCP_FLAG_SACK_PERM                0x02
+
+/* This sender sent FIN first */
+#define IP_CT_TCP_FLAG_CLOSE_INIT               0x04
+
+/* Be liberal in window checking */
+#define IP_CT_TCP_FLAG_BE_LIBERAL               0x08
+
 /* 
  * Old deprecated API, its use for new applications is *strongly discouraged* 
  */
@@ -558,116 +639,159 @@ typedef int (*nfct_callback)(void *arg, unsigned int flags, int, void *data);
 /*
  * [Allocate|free] a conntrack
  */
-extern struct nfct_conntrack *
+extern __attribute__((deprecated))
+struct nfct_conntrack *
 nfct_conntrack_alloc(struct nfct_tuple *orig, struct nfct_tuple *reply,
 		     u_int32_t timeout, union nfct_protoinfo *proto,
 		     u_int32_t status, u_int32_t mark,
 		     u_int32_t id, struct nfct_nat *range);
-extern void nfct_conntrack_free(struct nfct_conntrack *ct);
+extern __attribute__((deprecated))
+void nfct_conntrack_free(struct nfct_conntrack *ct);
 
 /*
  * [Allocate|free] an expectation
  */
-extern struct nfct_expect *
+extern __attribute__((deprecated))
+struct nfct_expect *
 nfct_expect_alloc(struct nfct_tuple *master, struct nfct_tuple *tuple,
 		  struct nfct_tuple *mask, u_int32_t timeout, 
 		  u_int32_t id);
-extern void nfct_expect_free(struct nfct_expect *exp);
+
+extern __attribute__((deprecated))
+void nfct_expect_free(struct nfct_expect *exp);
 
 
 /*
  * [Register|unregister] callbacks
  */
-extern void nfct_register_callback(struct nfct_handle *cth,
-				   nfct_callback callback, void *data);
-extern void nfct_unregister_callback(struct nfct_handle *cth);
+extern __attribute__((deprecated)) 
+void nfct_register_callback(struct nfct_handle *cth,
+			    nfct_callback callback, void *data);
+
+extern __attribute__((deprecated))
+void nfct_unregister_callback(struct nfct_handle *cth);
 
 /*
  * callback displayers
  */
-extern int nfct_default_conntrack_display(void *, unsigned int, int, void *); 
-extern int nfct_default_conntrack_display_id(void *, unsigned int, int, void *);
-extern int nfct_default_expect_display(void *, unsigned int, int, void *);
-extern int nfct_default_expect_display_id(void *, unsigned int, int, void *);
-extern int nfct_default_conntrack_event_display(void *, unsigned int, int, 
-						void *);
+extern __attribute__((deprecated)) 
+int nfct_default_conntrack_display(void *, unsigned int, int, void *); 
+extern __attribute__((deprecated))
+int nfct_default_conntrack_display_id(void *, unsigned int, int, void *);
+extern __attribute__((deprecated))
+int nfct_default_expect_display(void *, unsigned int, int, void *);
+extern __attribute__((deprecated))
+int nfct_default_expect_display_id(void *, unsigned int, int, void *);
+extern __attribute__((deprecated))
+int nfct_default_conntrack_event_display(void *, unsigned int, int, void *);
 
 /*
  * [Create|update|get|destroy] conntracks
  */
-extern int nfct_create_conntrack(struct nfct_handle *cth, 
-				 struct nfct_conntrack *ct);
-extern int nfct_update_conntrack(struct nfct_handle *cth,
-				 struct nfct_conntrack *ct);
-extern int nfct_delete_conntrack(struct nfct_handle *cth, 
-				 struct nfct_tuple *tuple, int dir, 
-				 u_int32_t id);
-extern int nfct_get_conntrack(struct nfct_handle *cth, 
-			      struct nfct_tuple *tuple, int dir,
-			      u_int32_t id); 
+extern __attribute__((deprecated))
+int nfct_create_conntrack(struct nfct_handle *cth, 
+			  struct nfct_conntrack *ct);
+extern __attribute__((deprecated))
+int nfct_update_conntrack(struct nfct_handle *cth,
+			  struct nfct_conntrack *ct);
+extern __attribute__((deprecated))
+int nfct_delete_conntrack(struct nfct_handle *cth, 
+			  struct nfct_tuple *tuple, int dir, 
+			  u_int32_t id);
+extern __attribute__((deprecated))
+int nfct_get_conntrack(struct nfct_handle *cth, 
+		       struct nfct_tuple *tuple, int dir,
+		       u_int32_t id); 
 /*
  * Conntrack table dumping & zeroing
  */
-extern int nfct_dump_conntrack_table(struct nfct_handle *cth, int family);
-extern int nfct_dump_conntrack_table_reset_counters(struct nfct_handle *cth, 
-						    int family);
+extern __attribute__((deprecated))
+int nfct_dump_conntrack_table(struct nfct_handle *cth, int family);
+extern __attribute__((deprecated))
+int nfct_dump_conntrack_table_reset_counters(struct nfct_handle *cth, 
+					     int family);
 
 /*
  * Conntrack event notification
  */
-extern int nfct_event_conntrack(struct nfct_handle *cth); 
+extern __attribute__((deprecated))
+int nfct_event_conntrack(struct nfct_handle *cth); 
 
 /*
  * Conntrack printing functions
  */
-extern int nfct_sprintf_conntrack(char *buf, struct nfct_conntrack *ct, 
-				  unsigned int flags);
-extern int nfct_sprintf_conntrack_id(char *buf, struct nfct_conntrack *ct,
-				     unsigned int flags);
-extern int nfct_sprintf_address(char *buf, struct nfct_tuple *t);
-extern int nfct_sprintf_proto(char *buf, struct nfct_tuple *t);
-extern int nfct_sprintf_protoinfo(char *buf, struct nfct_conntrack *ct);
-extern int nfct_sprintf_timeout(char *buf, struct nfct_conntrack *ct);
-extern int nfct_sprintf_protocol(char *buf, struct nfct_conntrack *ct);
-extern int nfct_sprintf_status_assured(char *buf, struct nfct_conntrack *ct);
-extern int nfct_sprintf_status_seen_reply(char *buf, struct nfct_conntrack *ct);
-extern int nfct_sprintf_counters(char *buf, struct nfct_conntrack *ct, int dir);
-extern int nfct_sprintf_mark(char *buf, struct nfct_conntrack *ct);
-extern int nfct_sprintf_use(char *buf, struct nfct_conntrack *ct);
-extern int nfct_sprintf_id(char *buf, u_int32_t id);
+extern __attribute__((deprecated))
+int nfct_sprintf_conntrack(char *buf, struct nfct_conntrack *ct, 
+			   unsigned int flags);
+extern __attribute__((deprecated))
+int nfct_sprintf_conntrack_id(char *buf, struct nfct_conntrack *ct,
+			      unsigned int flags);
+extern __attribute__((deprecated))
+int nfct_sprintf_address(char *buf, struct nfct_tuple *t);
+extern __attribute__((deprecated))
+int nfct_sprintf_proto(char *buf, struct nfct_tuple *t);
+extern __attribute__((deprecated))
+int nfct_sprintf_protoinfo(char *buf, struct nfct_conntrack *ct);
+extern __attribute__((deprecated))
+int nfct_sprintf_timeout(char *buf, struct nfct_conntrack *ct);
+extern __attribute__((deprecated))
+int nfct_sprintf_protocol(char *buf, struct nfct_conntrack *ct);
+extern __attribute__((deprecated))
+int nfct_sprintf_status_assured(char *buf, struct nfct_conntrack *ct);
+extern __attribute__((deprecated))
+int nfct_sprintf_status_seen_reply(char *buf, struct nfct_conntrack *ct);
+extern __attribute__((deprecated))
+int nfct_sprintf_counters(char *buf, struct nfct_conntrack *ct, int dir);
+extern __attribute__((deprecated))
+int nfct_sprintf_mark(char *buf, struct nfct_conntrack *ct);
+extern __attribute__((deprecated))
+int nfct_sprintf_use(char *buf, struct nfct_conntrack *ct);
+extern __attribute__((deprecated))
+int nfct_sprintf_id(char *buf, u_int32_t id);
 
 /*
  * Conntrack comparison
  */
-extern int nfct_conntrack_compare(struct nfct_conntrack *ct1, 
-				  struct nfct_conntrack *ct2,
-				  struct nfct_conntrack_compare *cmp);
+extern __attribute__((deprecated))
+int nfct_conntrack_compare(struct nfct_conntrack *ct1, 
+			   struct nfct_conntrack *ct2,
+			   struct nfct_conntrack_compare *cmp);
 
 /* 
  * Expectations
  */
-extern int nfct_dump_expect_list(struct nfct_handle *cth, int family);
-extern int nfct_flush_conntrack_table(struct nfct_handle *cth, int family);
-extern int nfct_get_expectation(struct nfct_handle *cth, 
-				struct nfct_tuple *tuple,
-				u_int32_t id);
-extern int nfct_create_expectation(struct nfct_handle *cth, struct nfct_expect *);
-extern int nfct_delete_expectation(struct nfct_handle *cth,
-				   struct nfct_tuple *tuple, u_int32_t id);
-extern int nfct_event_expectation(struct nfct_handle *cth);
-extern int nfct_flush_expectation_table(struct nfct_handle *cth, int family);
+extern __attribute__((deprecated))
+int nfct_dump_expect_list(struct nfct_handle *cth, int family);
+extern __attribute__((deprecated))
+int nfct_flush_conntrack_table(struct nfct_handle *cth, int family);
+extern __attribute__((deprecated))
+int nfct_get_expectation(struct nfct_handle *cth, 
+			 struct nfct_tuple *tuple,
+			 u_int32_t id);
+extern __attribute__((deprecated))
+int nfct_create_expectation(struct nfct_handle *cth, struct nfct_expect *);
+extern __attribute__((deprecated))
+int nfct_delete_expectation(struct nfct_handle *cth,
+			    struct nfct_tuple *tuple, u_int32_t id);
+extern __attribute__((deprecated))
+int nfct_event_expectation(struct nfct_handle *cth);
+extern __attribute__((deprecated))
+int nfct_flush_expectation_table(struct nfct_handle *cth, int family);
 
 /*
  * expectation printing functions
  */
-extern int nfct_sprintf_expect(char *buf, struct nfct_expect *exp);
-extern int nfct_sprintf_expect_id(char *buf, struct nfct_expect *exp);
+extern __attribute__((deprecated))
+int nfct_sprintf_expect(char *buf, struct nfct_expect *exp);
+extern __attribute__((deprecated))
+int nfct_sprintf_expect_id(char *buf, struct nfct_expect *exp);
 
 /*
  * low-level functions for libnetfilter_cthelper
  */
-extern void nfct_build_tuple(struct nfnlhdr *req, int size, 
-			     struct nfct_tuple *t, int type);
+extern __attribute__((deprecated))
+void nfct_build_tuple(struct nfnlhdr *req, int size, 
+		      struct nfct_tuple *t, int type);
 
 #ifdef __cplusplus
 }
