@@ -289,6 +289,8 @@ const void *nfct_get_attr(const struct nf_conntrack *ct,
 		return NULL;
 	}
 
+	assert(get_attr_array[type]);
+
 	return get_attr_array[type](ct);
 }
 
@@ -304,7 +306,7 @@ const void *nfct_get_attr(const struct nf_conntrack *ct,
 u_int8_t nfct_get_attr_u8(const struct nf_conntrack *ct,
 			  const enum nf_conntrack_attr type)
 {
-	const int *ret = nfct_get_attr(ct, type);
+	const u_int8_t *ret = nfct_get_attr(ct, type);
 	return ret == NULL ? 0 : *ret;
 }
 
@@ -320,7 +322,7 @@ u_int8_t nfct_get_attr_u8(const struct nf_conntrack *ct,
 u_int16_t nfct_get_attr_u16(const struct nf_conntrack *ct,
 			    const enum nf_conntrack_attr type)
 {
-	const int *ret = nfct_get_attr(ct, type);
+	const u_int16_t *ret = nfct_get_attr(ct, type);
 	return ret == NULL ? 0 : *ret;
 }
 
@@ -336,7 +338,7 @@ u_int16_t nfct_get_attr_u16(const struct nf_conntrack *ct,
 u_int32_t nfct_get_attr_u32(const struct nf_conntrack *ct,
 			    const enum nf_conntrack_attr type)
 {
-	const int *ret = nfct_get_attr(ct, type);
+	const u_int32_t *ret = nfct_get_attr(ct, type);
 	return ret == NULL ? 0 : *ret;
 }
 
@@ -548,7 +550,7 @@ int nfct_parse_conntrack(enum nf_conntrack_msg_type type,
 }
 
 /**
- * nfct_query - send a query to ctnetlin
+ * nfct_query - send a query to ctnetlink and handle the reply
  * @h: library handler
  * @qt: query type
  * @data: data required to send the query
@@ -561,18 +563,53 @@ int nfct_query(struct nfct_handle *h,
 	       const void *data)
 {
 	size_t size = 4096;	/* enough for now */
-	char buffer[4096];
-	struct nfnlhdr *req = (struct nfnlhdr *) buffer;
+	union {
+		char buffer[size];
+		struct nfnlhdr req;
+	} u;
 
 	assert(h != NULL);
 	assert(data != NULL);
 
-	if (nfct_build_query(h->nfnlssh_ct, qt, data, req, size) == -1)
+	if (nfct_build_query(h->nfnlssh_ct, qt, data, &u.req, size) == -1)
 		return -1;
 
-	return nfnl_query(h->nfnlh, &req->nlh);
+	return nfnl_query(h->nfnlh, &u.req.nlh);
 }
 
+/**
+ * nfct_send - send a query to ctnetlink
+ * @h: library handler
+ * @qt: query type
+ * @data: data required to send the query
+ *
+ * Like nfct_query but we do not wait for the reply from ctnetlink. 
+ * You can use nfct_send() and nfct_catch() to emulate nfct_query().
+ * This is particularly useful when the socket is non-blocking.
+ *
+ * On error, -1 is returned and errno is explicitely set. On success, 0
+ * is returned.
+ */
+int nfct_send(struct nfct_handle *h,
+	      const enum nf_conntrack_query qt,
+	      const void *data)
+{
+	size_t size = 4096;	/* enough for now */
+	union {
+		char buffer[size];
+		struct nfnlhdr req;
+	} u;
+
+	assert(h != NULL);
+	assert(data != NULL);
+
+	if (nfct_build_query(h->nfnlssh_ct, qt, data, &u.req, size) == -1)
+		return -1;
+
+	return nfnl_send(h->nfnlh, &u.req.nlh);
+}
+
+
 /**
  * nfct_catch - catch events
  * @h: library handler
@@ -607,8 +644,9 @@ int nfct_catch(struct nfct_handle *h)
  * 	- NFCT_O_XML: XML output
  *
  * The output flags are:
- * 	- NFCT_O_LAYER: include layer 3 information in the output, this is
- * 			*only* required by NFCT_O_DEFAULT.
+ * 	- NFCT_OF_SHOW_LAYER3: include layer 3 information in the output, 
+ * 	   this is *only* required by NFCT_O_DEFAULT.
+ * 	- NFCT_OF_TIME: display time.
  *
  * This function returns the size of the information that _would_ have been 
  * written to the buffer, even if there was no room for it. Thus, the
@@ -639,6 +677,8 @@ int nfct_snprintf(char *buf,
  *
  * If both conntrack object are equal, this function returns 1, otherwise
  * 0 is returned.
+ *
+ * NOTICE: The use nfct_cmp is preferred.
  */
 int nfct_compare(const struct nf_conntrack *ct1, 
 		 const struct nf_conntrack *ct2)
@@ -646,5 +686,159 @@ int nfct_compare(const struct nf_conntrack *ct1,
 	assert(ct1 != NULL);
 	assert(ct2 != NULL);
 
-	return __compare(ct1, ct2);
+	return __compare(ct1, ct2, NFCT_CMP_ALL);
+}
+
+/**
+ * nfct_cmp - compare two conntrack objects
+ * @ct1: pointer to a valid conntrack object
+ * @ct2: pointer to a valid conntrack object
+ * @flags: flags
+ *
+ * This function only compare attribute set in both objects, ie. if a certain
+ * attribute is not set in ct1 but it is in ct2, then the value of such 
+ * attribute is not used in the comparison.
+ *
+ * The available flags are:
+ *
+ * 	- NFCT_CMP_ALL: full comparison of both objects
+ * 	- NFCT_CMP_ORIG: it only compares the source and destination address;
+ * 	source and destination ports; the layer 3 and 4 protocol numbers
+ * 	of the original direction; and the id (if present).
+ * 	- NFCT_CMP_REPL: like NFCT_CMP_REPL but it compares the flow
+ * 	information that goes in the reply direction.
+ * 	- NFCT_CMP_TIMEOUT_EQ: timeout(ct1) == timeout(ct2)
+ * 	- NFCT_CMP_TIMEOUT_GT: timeout(ct1) > timeout(ct2)
+ * 	- NFCT_CMP_TIMEOUT_LT: timeout(ct1) < timeout(ct2)
+ * 	- NFCT_CMP_TIMEOUT_GE: timeout(ct1) >= timeout(ct2)
+ * 	- NFCT_CMP_TIMEOUT_LE: timeout(ct1) <= timeout(ct2)
+ *
+ * The default status bits comparison consists of the following operation:
+ * status(ct1) & status(ct2) == status(ct1).
+ *
+ * If both conntrack object are equal, this function returns 1, otherwise
+ * 0 is returned.
+ */
+int nfct_cmp(const struct nf_conntrack *ct1, 
+	     const struct nf_conntrack *ct2,
+	     unsigned int flags)
+{
+	assert(ct1 != NULL);
+	assert(ct2 != NULL);
+
+	return __compare(ct1, ct2, flags);
+}
+
+/**
+ * nfct_copy - copy part of one source object to another
+ * @ct1: destination object
+ * @ct2: source object
+ * @flags: flags
+ *
+ * This function copies one part of the source object to the target.
+ * It behaves like clone but:
+ *
+ * 1) You have to pass an already allocated space for the target object
+ * 2) You can copy only a part of the source object to the target
+ *
+ * The current supported flags are:
+ * 	- NFCT_CP_ALL that copies the object entirely.
+ * 	- NFCT_CP_ORIG and NFCT_CP_REPL that can be used to copy the
+ * 	information that identifies a flow in the original and the reply
+ * 	direction. This information is usually composed of: source and
+ * 	destination IP address; source and destination ports; layer 3
+ * 	and 4 protocol number.
+ * 	- NFCT_CP_META that copies the metainformation 
+ * 	  (all the attributes >= ATTR_TCP_STATE)
+ */
+void nfct_copy(struct nf_conntrack *ct1,
+	       const struct nf_conntrack *ct2,
+	       unsigned int flags)
+{
+	int i;
+
+	assert(ct1 != NULL);
+	assert(ct2 != NULL);
+
+	if (flags == NFCT_CP_ALL) {
+		for (i=0; i<ATTR_MAX; i++) {
+			if (test_bit(i, ct2->set)) {
+				copy_attr_array[i](ct1, ct2);
+				set_bit(i, ct1->set);
+			}
+		}
+		return;
+	}
+
+	static int cp_orig_mask[] = {
+		ATTR_ORIG_IPV4_SRC,
+		ATTR_ORIG_IPV4_DST,
+		ATTR_ORIG_IPV6_SRC,
+		ATTR_ORIG_IPV6_DST,
+		ATTR_ORIG_PORT_SRC,
+		ATTR_ORIG_PORT_DST,
+		ATTR_ICMP_TYPE,
+		ATTR_ICMP_CODE,
+		ATTR_ICMP_ID,
+		ATTR_ORIG_L3PROTO,
+		ATTR_ORIG_L4PROTO,
+	};
+	#define __CP_ORIG_MAX sizeof(cp_orig_mask)/sizeof(int)
+
+	if (flags & NFCT_CP_ORIG) {
+		for (i=0; i<__CP_ORIG_MAX; i++) {
+			if (test_bit(cp_orig_mask[i], ct2->set)) {
+				copy_attr_array[cp_orig_mask[i]](ct1, ct2);
+				set_bit(cp_orig_mask[i], ct1->set);
+			}
+		}
+	}
+
+	static int cp_repl_mask[] = {
+		ATTR_REPL_IPV4_SRC,
+		ATTR_REPL_IPV4_DST,
+		ATTR_REPL_IPV6_SRC,
+		ATTR_REPL_IPV6_DST,
+		ATTR_REPL_PORT_SRC,
+		ATTR_REPL_PORT_DST,
+		ATTR_REPL_L3PROTO,
+		ATTR_REPL_L4PROTO,
+	};
+	#define __CP_REPL_MAX sizeof(cp_repl_mask)/sizeof(int)
+
+	if (flags & NFCT_CP_REPL) {
+		for (i=0; i<__CP_REPL_MAX; i++) {
+			if (test_bit(cp_repl_mask[i], ct2->set)) {
+				copy_attr_array[cp_repl_mask[i]](ct1, ct2);
+				set_bit(cp_repl_mask[i], ct1->set);
+			}
+		}
+	}
+
+	if (flags & NFCT_CP_META) {
+		for (i=ATTR_TCP_STATE; i<ATTR_MAX; i++) {
+			if (test_bit(i, ct2->set)) {
+				copy_attr_array[i](ct1, ct2);
+				set_bit(i, ct1->set);
+			}
+		}
+	}
+}
+
+/**
+ * nfct_copy_attr - copy an attribute of one source object to another
+ * @ct1: destination object
+ * @ct2: source object
+ * @flags: flags
+ *
+ * This function copies one attribute (if present) to another object.
+ */
+void nfct_copy_attr(struct nf_conntrack *ct1,
+		    const struct nf_conntrack *ct2,
+		    const enum nf_conntrack_attr type)
+{
+	if (test_bit(type, ct2->set)) {
+		copy_attr_array[type](ct1, ct2);
+		set_bit(type, ct1->set);
+	}
 }
