@@ -25,6 +25,9 @@ static void __parse_ip(const struct nfattr *attr,
 		case __DIR_REPL:
 			set_bit(ATTR_REPL_IPV4_SRC, set);
 			break;
+		case __DIR_MASTER:
+			set_bit(ATTR_MASTER_IPV4_SRC, set);
+			break;
 		}
 	}
 
@@ -37,6 +40,9 @@ static void __parse_ip(const struct nfattr *attr,
 		case __DIR_REPL:
 			set_bit(ATTR_REPL_IPV4_DST, set);
 			break;
+		case __DIR_MASTER:
+			set_bit(ATTR_MASTER_IPV4_DST, set);
+			break;
 		}
 	}
 
@@ -50,6 +56,9 @@ static void __parse_ip(const struct nfattr *attr,
 		case __DIR_REPL:
 			set_bit(ATTR_REPL_IPV6_SRC, set);
 			break;
+		case __DIR_MASTER:
+			set_bit(ATTR_MASTER_IPV6_SRC, set);
+			break;
 		}
 	}
 
@@ -63,6 +72,9 @@ static void __parse_ip(const struct nfattr *attr,
 		case __DIR_REPL:
 			set_bit(ATTR_REPL_IPV6_DST, set);
 			break;
+		case __DIR_MASTER:
+			set_bit(ATTR_MASTER_IPV6_DST, set);
+			break;
 		}
 	}
 }
@@ -85,6 +97,9 @@ static void __parse_proto(const struct nfattr *attr,
 		case __DIR_REPL:
 			set_bit(ATTR_REPL_L4PROTO, set);
 			break;
+		case __DIR_MASTER:
+			set_bit(ATTR_MASTER_L4PROTO, set);
+			break;
 		}
 	}
 
@@ -98,6 +113,9 @@ static void __parse_proto(const struct nfattr *attr,
 		case __DIR_REPL:
 			set_bit(ATTR_REPL_PORT_SRC, set);
 			break;
+		case __DIR_MASTER:
+			set_bit(ATTR_MASTER_PORT_SRC, set);
+			break;
 		}
 	}
 	
@@ -111,6 +129,9 @@ static void __parse_proto(const struct nfattr *attr,
 		case __DIR_REPL:
 			set_bit(ATTR_REPL_PORT_DST, set);
 			break;
+		case __DIR_MASTER:
+			set_bit(ATTR_MASTER_PORT_DST, set);
+			break;
 		}
 	}
 	
@@ -131,6 +152,24 @@ static void __parse_proto(const struct nfattr *attr,
 			*(u_int16_t *)NFA_DATA(tb[CTA_PROTO_ICMP_ID-1]);
 		set_bit(ATTR_ICMP_ID, set);
 	}
+
+	if (tb[CTA_PROTO_ICMPV6_TYPE-1]) {
+		tuple->l4dst.icmp.type =
+			*(u_int8_t *)NFA_DATA(tb[CTA_PROTO_ICMPV6_TYPE-1]);
+		set_bit(ATTR_ICMP_TYPE, set);
+	}
+	
+	if (tb[CTA_PROTO_ICMPV6_CODE-1]) {
+		tuple->l4dst.icmp.code =
+			*(u_int8_t *)NFA_DATA(tb[CTA_PROTO_ICMPV6_CODE-1]);
+		set_bit(ATTR_ICMP_CODE, set);
+	}
+	
+	if (tb[CTA_PROTO_ICMPV6_ID-1]) {
+		tuple->l4src.icmp.id =
+			*(u_int16_t *)NFA_DATA(tb[CTA_PROTO_ICMPV6_ID-1]);
+		set_bit(ATTR_ICMP_ID, set);
+	}
 }
 
 void __parse_tuple(const struct nfattr *attr,
@@ -160,6 +199,49 @@ static void __parse_protoinfo_tcp(const struct nfattr *attr,
                         *(u_int8_t *)NFA_DATA(tb[CTA_PROTOINFO_TCP_STATE-1]);
 		set_bit(ATTR_TCP_STATE, ct->set);
 	}
+
+	if (tb[CTA_PROTOINFO_TCP_FLAGS_ORIGINAL-1]) {
+		memcpy(&ct->protoinfo.tcp.flags[0], 
+		       NFA_DATA(tb[CTA_PROTOINFO_TCP_FLAGS_ORIGINAL-1]),
+		       sizeof(u_int16_t));
+		set_bit(ATTR_TCP_FLAGS_ORIG, ct->set);
+		set_bit(ATTR_TCP_MASK_ORIG, ct->set);
+	}
+
+	if (tb[CTA_PROTOINFO_TCP_FLAGS_REPLY-1]) {
+		memcpy(&ct->protoinfo.tcp.flags[1], 
+		       NFA_DATA(tb[CTA_PROTOINFO_TCP_FLAGS_REPLY-1]),
+		       sizeof(u_int16_t));
+		set_bit(ATTR_TCP_FLAGS_REPL, ct->set);
+		set_bit(ATTR_TCP_MASK_REPL, ct->set);
+	}
+}
+
+static void __parse_protoinfo_sctp(const struct nfattr *attr, 
+				   struct nf_conntrack *ct)
+{
+	struct nfattr *tb[CTA_PROTOINFO_SCTP_MAX];
+
+	nfnl_parse_nested(tb, CTA_PROTOINFO_SCTP_MAX, attr);
+
+	if (tb[CTA_PROTOINFO_SCTP_STATE-1]) {
+                ct->protoinfo.sctp.state =
+                        *(u_int8_t *)NFA_DATA(tb[CTA_PROTOINFO_SCTP_STATE-1]);
+		set_bit(ATTR_SCTP_STATE, ct->set);
+	}
+
+	if (tb[CTA_PROTOINFO_SCTP_VTAG_ORIGINAL-1]) {
+		ct->protoinfo.sctp.vtag[__DIR_ORIG] = 
+			ntohl(*(u_int32_t *)NFA_DATA(tb[CTA_PROTOINFO_SCTP_VTAG_ORIGINAL-1]));
+		set_bit(ATTR_SCTP_VTAG_ORIG, ct->set);
+	}
+
+	if (tb[CTA_PROTOINFO_SCTP_VTAG_REPLY-1]) {
+		ct->protoinfo.sctp.vtag[__DIR_ORIG] = 
+			ntohl(*(u_int32_t *)NFA_DATA(tb[CTA_PROTOINFO_SCTP_VTAG_REPLY-1]));
+		set_bit(ATTR_SCTP_VTAG_ORIG, ct->set);
+	}
+
 }
 
 static void __parse_protoinfo(const struct nfattr *attr,
@@ -169,10 +251,11 @@ static void __parse_protoinfo(const struct nfattr *attr,
 
 	nfnl_parse_nested(tb, CTA_PROTOINFO_MAX, attr);
 
-	if (!tb[CTA_PROTOINFO_TCP-1])
-		return;
+	if (tb[CTA_PROTOINFO_TCP-1])
+		__parse_protoinfo_tcp(tb[CTA_PROTOINFO_TCP-1], ct);
 
-	__parse_protoinfo_tcp(tb[CTA_PROTOINFO_TCP-1], ct);
+	if (tb[CTA_PROTOINFO_SCTP-1])
+		__parse_protoinfo_sctp(tb[CTA_PROTOINFO_SCTP-1], ct);
 }
 
 static void __parse_counters(const struct nfattr *attr,
@@ -226,6 +309,53 @@ static void __parse_counters(const struct nfattr *attr,
 	}
 }
 
+static void 
+__parse_nat_seq(const struct nfattr *attr, struct nf_conntrack *ct, int dir)
+{
+	struct nfattr *tb[CTA_NAT_SEQ_MAX];
+
+	nfnl_parse_nested(tb, CTA_NAT_SEQ_MAX, attr);
+
+	if (tb[CTA_NAT_SEQ_CORRECTION_POS-1]) {
+		ct->tuple[dir].natseq.correction_pos =
+		ntohl(*(u_int32_t *)NFA_DATA(tb[CTA_NAT_SEQ_CORRECTION_POS-1]));
+		switch(dir) {
+		case __DIR_ORIG:
+			set_bit(ATTR_ORIG_NAT_SEQ_CORRECTION_POS, ct->set);
+			break;
+		case __DIR_REPL:
+			set_bit(ATTR_REPL_NAT_SEQ_CORRECTION_POS, ct->set);
+			break;
+		}
+	}
+					
+	if (tb[CTA_NAT_SEQ_OFFSET_BEFORE-1]) {
+		ct->tuple[dir].natseq.offset_before =
+		ntohl(*(u_int32_t *)NFA_DATA(tb[CTA_NAT_SEQ_OFFSET_BEFORE-1]));
+		switch(dir) {
+		case __DIR_ORIG:
+			set_bit(ATTR_ORIG_NAT_SEQ_OFFSET_BEFORE, ct->set);
+			break;
+		case __DIR_REPL:
+			set_bit(ATTR_REPL_NAT_SEQ_OFFSET_BEFORE, ct->set);
+			break;
+		}
+	}
+
+	if (tb[CTA_NAT_SEQ_OFFSET_AFTER-1]) {
+		ct->tuple[dir].natseq.offset_after =
+		ntohl(*(u_int32_t *)NFA_DATA(tb[CTA_NAT_SEQ_OFFSET_AFTER-1]));
+		switch(dir) {
+		case __DIR_ORIG:
+			set_bit(ATTR_ORIG_NAT_SEQ_OFFSET_AFTER, ct->set);
+			break;
+		case __DIR_REPL:
+			set_bit(ATTR_REPL_NAT_SEQ_OFFSET_AFTER, ct->set);
+			break;
+		}
+	}
+}
+
 int __parse_message_type(const struct nlmsghdr *nlh)
 {
 	u_int16_t type = NFNL_MSG_TYPE(nlh->nlmsg_type);
@@ -249,19 +379,35 @@ void __parse_conntrack(const struct nlmsghdr *nlh,
 {
 	struct nfgenmsg *nfhdr = NLMSG_DATA(nlh);
 
-	ct->tuple[__DIR_ORIG].l3protonum = nfhdr->nfgen_family;
-	set_bit(ATTR_ORIG_L3PROTO, ct->set);
+	if (cda[CTA_TUPLE_ORIG-1]) {
+		ct->tuple[__DIR_ORIG].l3protonum = nfhdr->nfgen_family;
+		set_bit(ATTR_ORIG_L3PROTO, ct->set);
 
-	ct->tuple[__DIR_REPL].l3protonum = nfhdr->nfgen_family;
-	set_bit(ATTR_REPL_L3PROTO, ct->set);
-
-	if (cda[CTA_TUPLE_ORIG-1])
 		__parse_tuple(cda[CTA_TUPLE_ORIG-1], 
 			      &ct->tuple[__DIR_ORIG], __DIR_ORIG, ct->set);
+	}
+
+	if (cda[CTA_TUPLE_REPLY-1]) {
+		ct->tuple[__DIR_REPL].l3protonum = nfhdr->nfgen_family;
+		set_bit(ATTR_REPL_L3PROTO, ct->set);
 
-	if (cda[CTA_TUPLE_REPLY-1])
 		__parse_tuple(cda[CTA_TUPLE_REPLY-1], 
 			      &ct->tuple[__DIR_REPL], __DIR_REPL, ct->set);
+	}
+
+	if (cda[CTA_TUPLE_MASTER-1]) {
+		ct->tuple[__DIR_MASTER].l3protonum = nfhdr->nfgen_family;
+		set_bit(ATTR_MASTER_L3PROTO, ct->set);
+
+		__parse_tuple(cda[CTA_TUPLE_MASTER-1], 
+			      &ct->tuple[__DIR_MASTER], __DIR_MASTER, ct->set);
+	}
+
+	if (cda[CTA_NAT_SEQ_ADJ_ORIG-1])
+		__parse_nat_seq(cda[CTA_NAT_SEQ_ADJ_ORIG-1], ct, __DIR_ORIG);
+
+	if (cda[CTA_NAT_SEQ_ADJ_REPLY-1])
+		__parse_nat_seq(cda[CTA_NAT_SEQ_ADJ_REPLY-1], ct, __DIR_REPL);
 
 	if (cda[CTA_STATUS-1]) {
 		ct->status = ntohl(*(u_int32_t *)NFA_DATA(cda[CTA_STATUS-1]));
@@ -281,6 +427,11 @@ void __parse_conntrack(const struct nlmsghdr *nlh,
 		set_bit(ATTR_MARK, ct->set);
 	}
 
+	if (cda[CTA_SECMARK-1]) {
+		ct->secmark = ntohl(*(u_int32_t *)NFA_DATA(cda[CTA_SECMARK-1]));
+		set_bit(ATTR_SECMARK, ct->set);
+	}
+
 	if (cda[CTA_COUNTERS_ORIG-1])
 		__parse_counters(cda[CTA_COUNTERS_ORIG-1], ct, __DIR_ORIG);
 
