@@ -45,6 +45,8 @@
  * 		</layer4>
  * 		<timeout>100</timeout>
  * 		<mark>1</mark>
+ * 		<secmark>0</secmark>
+ * 		<id>453281439</id>
  * 		<use>1</use>
  * 		<assured/>
  * 	</meta>
@@ -54,7 +56,9 @@
 static char *proto2str[IPPROTO_MAX] = {
 	[IPPROTO_TCP] = "tcp",
         [IPPROTO_UDP] = "udp",
+        [IPPROTO_UDPLITE] = "udplite",
         [IPPROTO_ICMP] = "icmp",
+        [IPPROTO_ICMPV6] = "icmp6",
         [IPPROTO_SCTP] = "sctp"
 };
 static char *l3proto2str[AF_MAX] = {
@@ -98,7 +102,7 @@ static int __snprintf_ipv6_xml(char *buf,
 	static char tmp[INET6_ADDRSTRLEN];
 	const void *p = (type == __ADDR_SRC) ? &tuple->src.v6 : &tuple->dst.v6;
 
-	memcpy(&addr.in6_u, p, sizeof(struct in6_addr));
+	memcpy(&addr, p, sizeof(struct in6_addr));
 
 	if (!inet_ntop(AF_INET6, &addr, tmp, sizeof(tmp)))
 		return -1;
@@ -161,6 +165,7 @@ static int __snprintf_proto_xml(char *buf,
 	switch(tuple->protonum) {
 	case IPPROTO_TCP:
 	case IPPROTO_UDP:
+	case IPPROTO_UDPLITE:
 	case IPPROTO_SCTP:
 		if (type == __ADDR_SRC) {
 			ret = snprintf(buf, len, "<sport>%u</sport>", 
@@ -307,11 +312,22 @@ int __snprintf_conntrack_xml(char *buf,
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
+	if (test_bit(ATTR_SECMARK, ct->set)) {
+		ret = snprintf(buf+offset, len, 
+				"<secmark>%u</secmark>", ct->secmark);
+		BUFFER_SIZE(ret, size, len, offset);
+	}
+
 	if (test_bit(ATTR_USE, ct->set)) {
 		ret = snprintf(buf+offset, len, "<use>%u</use>", ct->use);
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
+	if (test_bit(ATTR_ID, ct->set)) {
+		ret = snprintf(buf+offset, len, "<id>%u</id>", ct->id);
+		BUFFER_SIZE(ret, size, len, offset);
+	}
+
 	if (test_bit(ATTR_STATUS, ct->set)
 	    && ct->status & IPS_ASSURED) {
 		ret = snprintf(buf+offset, len, "<assured/>");
@@ -332,6 +348,46 @@ int __snprintf_conntrack_xml(char *buf,
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
+	if (flags & NFCT_OF_TIME) {
+		time_t t;
+		struct tm tm;
+
+		t = time(NULL);
+		if (localtime_r(&t, &tm) == NULL)
+			goto err_out;
+
+		ret = snprintf(buf+offset, len, "<when>");
+		BUFFER_SIZE(ret, size, len, offset);
+
+		ret = snprintf(buf+offset, len, "<hour>%d</hour>", tm.tm_hour);
+		BUFFER_SIZE(ret, size, len, offset);
+
+		ret = snprintf(buf+offset, len, "<min>%02d</min>", tm.tm_min);
+		BUFFER_SIZE(ret, size, len, offset);
+
+		ret = snprintf(buf+offset, len, "<sec>%02d</sec>", tm.tm_sec);
+		BUFFER_SIZE(ret, size, len, offset);
+
+		ret = snprintf(buf+offset, len, "<wday>%d</wday>", 
+			       tm.tm_wday + 1);
+		BUFFER_SIZE(ret, size, len, offset);
+
+		ret = snprintf(buf+offset, len, "<day>%d</day>", tm.tm_mday);
+		BUFFER_SIZE(ret, size, len, offset);
+
+		ret = snprintf(buf+offset, len, "<month>%d</month>", 
+			       tm.tm_mon + 1);
+		BUFFER_SIZE(ret, size, len, offset);
+
+		ret = snprintf(buf+offset, len, "<year>%d</year>", 
+			       1900 + tm.tm_year);
+		BUFFER_SIZE(ret, size, len, offset);
+
+		ret = snprintf(buf+offset, len, "</when>");
+		BUFFER_SIZE(ret, size, len, offset);
+	}
+
+err_out:
 	ret = snprintf(buf+offset, len, "</flow>");
 	BUFFER_SIZE(ret, size, len, offset);
 
